<?php

namespace Drupal\geolocation_google_maps\Plugin\geolocation\MapFeature;

use Drupal\Core\Form\FormStateInterface;
use Drupal\geolocation\MapFeatureBase;
use Drupal\geolocation_google_maps\Plugin\geolocation\MapProvider\GoogleMaps;

/**
 * Class ControlMapFeatureBase.
 */
abstract class ControlMapFeatureBase extends MapFeatureBase {

  /**
   * {@inheritdoc}
   */
  public static function getDefaultSettings() {
    return [
      'position' => 'TOP_LEFT',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getSettingsForm(array $settings, array $parents) {
    $form = [];

    $settings = array_replace_recursive(
      self::getDefaultSettings(),
      $settings
    );

    $form['position'] = [
      '#type' => 'select',
      '#title' => $this->t('Position'),
      '#options' => GoogleMaps::getControlPositions(),
      '#default_value' => $settings['position'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateSettingsForm(array $values, FormStateInterface $form_state, array $parents) {
    if (!in_array($values['position'], array_keys(GoogleMaps::getControlPositions()))) {
      $form_state->setErrorByName(implode('][', $parents), $this->t('No valid position.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function alterMap(array $render_array, array $feature_settings) {
    $render_array['#controls'][$this->pluginId] = [
      '#position' => $feature_settings['position'],
    ];

    return $render_array;
  }

}
